(**************************************************************************)
(*                                                                        *)
(*  FORK                                                                  *)
(*                                                                        *)
(*  Author:  Franois Pottier, INRIA Paris-Rocquencourt                   *)
(*  Version: 20101109                                                     *)
(*                                                                        *)
(*  The copyright to this code is held by Institut National de Recherche  *)
(*  en Informatique et en Automatique (INRIA). All rights reserved. This  *)
(*  file is distributed under the license CeCILL-C (see file LICENSE).    *)
(*                                                                        *)
(**************************************************************************)

{

  open Lexing
  open Parser

  (* A table of keywords. *)

  let string2keyword, keyword2string =
    LexerUtil.setup [
      "default", DEFAULT;
      "kind", KIND;
      "type", TYPE;
      "later", LATER;
      "lemma", LEMMA;
      "forall", FORALL;
      "exists", EXISTS;
      "term", TERM;
      "pack", PACK;
      "in", IN;
      "unpack", UNPACK;
      "as", AS;
      "let", LET;
      "assert", ASSERT;
    ]

}

let newline = ('\010' | '\013' | "\013\010")

let whitespace = [ ' ' '\t' ]

let digit = [ '0'-'9' ]

let integer = ( "0x" | "0o" | "0b" )? digit+

let identchar = ['A'-'Z' 'a'-'z' '_' '\192'-'\214' '\216'-'\246' '\248'-'\255' '\'' '0'-'9' '@']

let identifier = identchar+

rule main = parse
| "include" whitespace+ '"' ( [^ '"']* as filename ) '"'
    { INCLUDE filename }
| newline
    { LexerUtil.newline lexbuf; main lexbuf }
| whitespace+
    { main lexbuf }
| "*"
    { STAR }
| "="
    { EQ }
| "("
    { LPAR }
| ")"
    { RPAR }
| "["
    { LBRACKET }
| "]"
    { RBRACKET }
| "->"
    { ARROW }
| "."
    { DOT }
| ","
    { COMMA }
| "`"
    { BACKQUOTE }
| ":"
    { COLON }
| "\\"
    { LAMBDA }
| "/\\"
    { BIGLAMBDA }
| "_"
    { WILDCARD }
| identifier as id
    { try
        string2keyword id
      with Not_found ->
	IDENTIFIER (id, lexeme_start_p lexbuf, lexeme_end_p lexbuf)
    }
| "(*"
    { comment (lexeme_start_p lexbuf) lexbuf; main lexbuf }
| eof
    { EOF }
| _
    { Error.errorb lexbuf "Unexpected character." }

and comment openingp = parse
| "(*"
    { comment (lexeme_start_p lexbuf) lexbuf; comment openingp lexbuf }
| "*)"
    { () }
| newline
    { LexerUtil.newline lexbuf; comment openingp lexbuf }
| eof
    { Error.error [ (openingp, lexeme_start_p lexbuf) ] "Unterminated comment." }
| _
    { comment openingp lexbuf }

